#
# mssql_secured::controls::asymmetric_key_size_is_set_to_greater_than_or_equal_to_2048_in_non_system_databases
#
# @summary Microsoft Best Practices recommend to use at least a 2048-bit encryption algorithm for asymmetric keys.
# The RSA_2048 encryption algorithm for asymmetric keys in SQL Server is the highest bit-level provided and therefore the most secure available choice (other choices are RSA_512 and RSA_1024).
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::asymmetric_key_size_is_set_to_greater_than_or_equal_to_2048_in_non_system_databases: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::asymmetric_key_size_is_set_to_greater_than_or_equal_to_2048_in_non_system_databases::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `asymmetric_key_size_is_set_to_greater_than_or_equal_to_2048_in_non_system_databases` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 7.2
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 7.2
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 7.2
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 7.2
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::asymmetric_key_size_is_set_to_greater_than_or_equal_to_2048_in_non_system_databases {
  mssql_secured_setup { "asymmetric_key_size_is_set_to_greater_than_or_equal_to_2048_in_non_system_databases on ${title}":
    ensure => 'present',
  }
  $instance = $title
  $asymmetric_keys = dig($facts, 'mssql_asymmetric_keys_in_non_system_databases', $instance)
  unless (empty($asymmetric_keys)) {
    $asymmetric_keys.each |$key| {
      alert("asymmetric key ${key} length is less then 2048 bits. Use another mey with length 2048 or more.")
    }
  }
}
