#
# mssql_secured::controls::auto_close_is_set_to_off_on_contained_databases
#
# @summary AUTO_CLOSE determines if a given database is closed or not after a connection terminates.
# If enabled, subsequent connections to the given database will require the database to be reopened and relevant procedure caches to be rebuilt.
# 
# Because authentication of users for contained databases occurs within the database not at the server\instance level, the database must be opened every time to authenticate a user. The frequent opening/closing of the database consumes additional server resources and may contribute to a denial of service.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::auto_close_is_set_to_off_on_contained_databases: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::auto_close_is_set_to_off_on_contained_databases::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `auto_close_is_set_to_off_on_contained_databases` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.16
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.16
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.16
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.15
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Array] exclude
#    The objects to exclude from the control actions.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::auto_close_is_set_to_off_on_contained_databases (
  Array $exclude = mssql_secured::lookup_setting('exclude', []),
) {
  $instance = $title

  mssql_secured_setup { "auto_close_is_set_to_off_on_contained_databases on ${title}":
    ensure => 'present',
  }

  $databases_to_check = dig($facts, 'mssql_databases', $instance) - ['master', 'tempdb', 'model', 'msdb'] - $exclude
  unless( empty($databases_to_check)) {
    $databases_to_check.each |$database| {
      sqlserver_tsql { "auto_close_is_set_to_off_on_contained_databases on ${title}/${database}":
        instance => $title,
        # lint:ignore:strict_indent
        command  => @("SQL"),
          ALTER DATABASE [${database}] SET AUTO_CLOSE Off
        SQL
        onlyif   => mssql_secured::exists(@("SQL")),
            SELECT name, containment, containment_desc, is_auto_close_on FROM sys.databases
            WHERE containment <> 0 
            and is_auto_close_on = 1
            and name = '${database}'
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["auto_close_is_set_to_off_on_contained_databases on ${title}"],
      }
    }
  }
}
