#
# mssql_secured::controls::check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role
#
# @summary Applies the same password expiration policy used in Windows to passwords used inside SQL Server.
# Ensuring SQL logins comply with the secure password policy applied by the Windows Server Benchmark will ensure the passwords for SQL logins with sysadmin privileges are changed on a frequent basis to help prevent compromise via a brute force attack. CONTROL SERVER is an equivalent permission to sysadmin and logins with that permission should also be required to have expiring passwords.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 4.2
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 4.2
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 4.2
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 4.2
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role {
  $instance = $title
  $sql_authenticated_logins_within_the_sysadmin_role = dig($facts, 'mssql_sql_authenticated_logins_within_the_sysadmin_role', $instance)

  mssql_secured_setup { "check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role on ${title}":
    ensure  => 'present',
    require => Mssql_secured_setup["check_policy_option_is_set_to_on_for_all_sql_authenticated_logins on ${title}"],
  }
  unless( empty($sql_authenticated_logins_within_the_sysadmin_role)) {
    $sql_authenticated_logins_within_the_sysadmin_role.each | String $login | {
      sqlserver_tsql { "check_expiration_option_is_set_to_on on ${login} on ${title}":
        instance => $instance,
        # lint:ignore:strict_indent
        command  => @("SQL"),
          ALTER LOGIN ${login} WITH CHECK_EXPIRATION = ON;
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role on ${title}"],
      }
    }
  }
}
