#
# mssql_secured::controls::check_policy_option_is_set_to_on_for_all_sql_authenticated_logins
#
# @summary Applies the same password complexity policy used in Windows to passwords used inside SQL Server.
# Ensure SQL authenticated login passwords comply with the secure password policy applied by the Windows Server Benchmark so that they cannot be easily compromised via brute force attack.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::check_policy_option_is_set_to_on_for_all_sql_authenticated_logins: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::check_policy_option_is_set_to_on_for_all_sql_authenticated_logins::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `check_policy_option_is_set_to_on_for_all_sql_authenticated_logins` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 4.3
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 4.3
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 4.3
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 4.3
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::check_policy_option_is_set_to_on_for_all_sql_authenticated_logins {
  $instance = $title
  $sql_authenticated_logins = dig($facts, 'mssql_sql_authenticated_logins_within_the_sysadmin_role', $instance)
  mssql_secured_setup { "check_policy_option_is_set_to_on_for_all_sql_authenticated_logins on ${title}":
    ensure => 'present',
  }
  unless( empty($sql_authenticated_logins)) {
    $sql_authenticated_logins.each | String $login | {
      sqlserver_tsql { "check_policy_option_is_set_to_on_for_all_sql_authenticated_logins on ${login} on ${title}":
        instance => $instance,
        # lint:ignore:strict_indent
        command  => @("SQL"),
          ALTER LOGIN ${login} WITH CHECK_POLICY = ON;
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["check_expiration_option_is_set_to_on_for_all_sql_authenticated_logins_within_the_sysadmin_role on ${title}"],
      }
    }
  }
}
