#
# mssql_secured::controls::clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies
#
# @summary Setting CLR Assembly Permission Sets to SAFE_ACCESS will prevent assemblies from accessing external system resources such as files, the network, environment variables, or the registry.
# Assemblies with EXTERNAL_ACCESS or UNSAFE permission sets can be used to access sensitive areas of the operating system, steal and/or transmit data and alter the state and other protection measures of the underlying Windows Operating System.
# 
# Assemblies which are Microsoft-created (is_user_defined = 0) are excluded from this check as they are required for overall system functionality.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 6.2
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 6.2
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 6.2
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 6.2
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies {
  $instance = $title
  $unsafe_assemblies = dig($facts, 'mssql_unsafe_assemblies', $instance)
  mssql_secured_setup { "clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies on ${title}":
    ensure => 'present',
  }
  unless( empty($unsafe_assemblies)) {
    $unsafe_assemblies.each |$assembly| {
      sqlserver_tsql { "clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies on ${title}/${assembly}":
        instance => $title,
        # lint:ignore:strict_indent
        command  => @("SQL"),
          ALTER ASSEMBLY ${assembly} WITH PERMISSION_SET = SAFE;
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["clr_assembly_permission_set_is_set_to_safe_access_for_all_clr_assemblies on ${title}"],
      }
    }
  }
}
