#
# mssql_secured::controls::connect_perm_on_guest_user_is_revoked_within_all_sql_server_dbs_excluding_master_msdb_and_tempdb
#
# @summary Remove the right of the guest user to connect to SQL Server databases, except for master, msdb, and tempdb.
# A login assumes the identity of the guest user when a login has access to SQL Server but does not have access to a database through its own account and the database has a guest user account. Revoking the CONNECT permission for the guest user will ensure that a login is not able to access database information without explicit access to do so.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::connect_perm_on_guest_user_is_revoked_within_all_sql_server_dbs_excluding_master_msdb_and_tempdb: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::connect_perm_on_guest_user_is_revoked_within_all_sql_server_dbs_excluding_master_msdb_and_tempdb::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `connect_perm_on_guest_user_is_revoked_within_all_sql_server_dbs_excluding_master_msdb_and_tempdb` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.2
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Array] exclude
#    The objects to exclude from the control actions.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::connect_permissions_on_the_guest_user_is_revoked_within_all_sql_server_databases_excluding_the_master_msdb_and_tempdb (
  Array $exclude = mssql_secured::lookup_setting('exclude', []),
) {
  $instance = $title

  mssql_secured_setup { "connect_permissions_on_the_guest_user_is_revoked_within_all_sql_server_databases_excluding_the_master_msdb_and_tempdb on ${title}":
    ensure => 'present',
  }

  $databases_to_check = dig($facts, 'mssql_databases', $instance) - ['master', 'tempdb', 'msdb'] - $exclude
  unless( empty($databases_to_check)) {
    $databases_to_check.each |$database| {
      sqlserver_tsql { "connect_permissions_on_the_guest_user_is_revoked_within_all_sql_server_databases_excluding_the_master_msdb_and_tempdb on ${title}/${database}":
        instance => $title,
        database => $database,
        # lint:ignore:strict_indent
        command  => @("SQL"),
           REVOKE CONNECT FROM guest
        SQL
        onlyif   => mssql_secured::exists(@("SQL")),
          SELECT DB_NAME() AS DatabaseName, 'guest' AS Database_User, [permission_name], [state_desc]
          FROM sys.database_permissions
          WHERE [grantee_principal_id] = DATABASE_PRINCIPAL_ID('guest') AND [state_desc] LIKE 'GRANT%'
          AND [permission_name] = 'CONNECT'
          AND DB_NAME() NOT IN ('master','tempdb','msdb')
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["connect_permissions_on_the_guest_user_is_revoked_within_all_sql_server_databases_excluding_the_master_msdb_and_tempdb on ${title}"],
      }
    }
  }
}
