#
# mssql_secured::controls::cross_db_ownership_chaining_server_configuration_option_is_set_to0
#
# @summary The cross db ownership chaining option controls cross-database ownership chaining across all databases at the instance (or server) level.
# When enabled, this option allows a member of the db_owner role in a database to gain access to objects owned by a login in any other database, causing an unnecessary information disclosure. When required, cross-database ownership chaining should only be enabled for the specific databases requiring it instead of at the instance level for all databases by using the ALTER DATABASE<database_name>SET DB_CHAINING ON command. This database option may not be changed on the master, model, or tempdb system databases.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::cross_db_ownership_chaining_server_configuration_option_is_set_to0: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::cross_db_ownership_chaining_server_configuration_option_is_set_to0::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `cross_db_ownership_chaining_server_configuration_option_is_set_to0` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.3
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.3
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.3
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.3
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::cross_db_ownership_chaining_server_configuration_option_is_set_to0 {
  $instance = $title
  $setting  =  'cross MSSQLSERVER ownership chaining'
  $hostname = dig($facts, 'hostname')
  $temp_dir = mssql_secured::env('TEMP')
  $system_root_dir = mssql_secured::env('SystemRoot')
  $title_without_whitespaces=regsubst($title,'\s','')
  $setting_without_whitespaces=regsubst($setting,'\s','')
  mssql_secured_setup { "cross_db_ownership_chaining_server_configuration_option_is_set_to0 on ${title}":
    ensure => 'present',
  }
  -> file { "${temp_dir}\\dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}.ps1":
    ensure  => 'file',
    force   => true,
    content => epp('mssql_secured/dsc_sqlconfiguration_optionn_template.epp',{
        'name'            => "${title}/${setting}",
        'title'           => $title,
        'instance'        => $instance,
        'hostname'        => $hostname,
        'dsc_optionname'  => $setting,
        'dsc_optionvalue' => 0,
        'restart_service' => false
    }),
  }
  -> exec { "Exec dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}ps1":
    command  => "dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
  -> exec { "Cleanup dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}ps1":
    command  => "Remove-Item -Recurse -Force unnecessary_sql_server_protocols_are_set_to_disabled${title_without_whitespaces}${setting_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
}
