#
# mssql_secured::controls::hide_instance_option_is_set_to_yes_for_production_sql_server_instances
#
# @summary Non-clustered SQL Server instances within production environments should be designated as hidden to prevent advertisement by the SQL Server Browser service.
# Designating production SQL Server instances as hidden leads to a more secure installation because they cannot be enumerated. However, clustered instances may break if this option is selected.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::hide_instance_option_is_set_to_yes_for_production_sql_server_instances: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::hide_instance_option_is_set_to_yes_for_production_sql_server_instances::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `hide_instance_option_is_set_to_yes_for_production_sql_server_instances` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.12
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.12
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.12
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.12
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::hide_instance_option_is_set_to_yes_for_production_sql_server_instances {
  $key = $facts.dig('mssql_instances', $title)

  mssql_secured_setup { "hide_instance_option_is_set_to_yes_for_production_sql_server_instances on ${title}":
    ensure => 'present',
  }

  -> registry_value { "hide_instance_option_is_set_to_yes_for_production_sql_server_instances on ${title}":
    ensure => present,
    path   => "HKLM\\SOFTWARE\\Microsoft\\Microsoft SQL Server\\${key}\\MSSQLServer\\SuperSocketNetLib\\HideInstance",
    type   => 'dword',
    data   => 1,
  }
}
