#
# mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed
#
# @summary SQL Server patches contain program updates that fix security and product functionality issues found in the software.
# These patches can be installed with a security update, which is a single patch, or a cumulative update which is a group of patches. The SQL Server version and patch levels should be the most recent compatible with the organizations' operational needs.
# 
# Using the most recent SQL Server software, along with all applicable patches can help limit the possibilities for vulnerabilities in the software. The installation version and/or patches applied during setup should be established according to the needs of the organization.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `latest_sql_server_service_packs_and_hotfixes_are_installed` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 1.1
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Hash] product_versions
#    This is Hash containing Windows Server versions and required updates.
#    Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed::product_versions:
#      15: 
#        latest_version: 15.0.4198.2
#        kb_update: KB5003279
#        sp_update: null
#      14: 
#        latest_version: 14.0.3430.2
#        kb_update: KB5003279
#        sp_update: null
#      13: 
#        latest_version: 13.0.6300.2
#        kb_update: KB5003279
#        sp_update: sqlserver2016sp3
#      13: 
#        latest_version: 13.0.6300.2
#        kb_update: KB5003279
#        sp_update: sqlserver2016sp3
#      12: 
#        latest_version: 12.0.6329.1
#        kb_update: KB5008084
#        sp_update: sqlserver2014sp3
#        version_numbers_without_servicepack: 12.0.6
#    ```
#
# @param [Hash] update_paths
#    This is Hash containing update, and update path.
#    Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed::update_paths:
#      KB5003279: "C:\\vagrant\\modules\\software\\files\\SQLServer2019-KB5008996-x64.exe"
#    ```
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed (
  Hash $update_paths = mssql_secured::lookup_setting('update_paths', []),
  Hash $product_versions = mssql_secured::lookup_setting('product_versions', []),
) {
  $instance = $title
  $product_version_fact = dig($facts, 'mssql_product_version', $instance)
  $major_version_fact = dig($facts, 'mssql_major_version', $instance)

  if $product_version_fact {
    $product_version = $product_version_fact[0]
  } else {
    $product_version = Undef
  }

  if $major_version_fact {
    $major_version = $major_version_fact[0]
  } else {
    $major_version = Undef
  }

  $updates = $product_versions[Integer($major_version)]
  unless($updates) {
    $version_fact = dig($facts, 'mssql_version', $instance)
    if $version_fact {
      $version = $version_fact[0]
    } else {
      $version = Undef
    }
    fail("SQLServer version: ${product_version} not supported. Version: ${version}")
  }

  unless((versioncmp($major_version, '12') == 0) or ($product_version == $updates['latest_version'])) {
    $kb = $updates['kb_update']
    $sp = $updates['sp_update']
  } else {
    if($product_version == $updates['latest_version']) {
      $kb = ''
      $sp = ''
    } else {
      if (versioncmp($product_version, $updates['version_numbers_without_servicepack']) > -1) {
        $kb = $updates['kb_update']
        $sp = ''
      } else {
        $kb = $updates['kb_update']
        $sp = $updates['sp_update']
      }
    }
  }

  mssql_secured_setup { "latest_sql_server_service_packs_and_hotfixes_are_installed on ${title}":
    ensure => 'present',
  }

  unless(empty($kb)) {
    $path_kb = dig($update_paths, $kb)
    package { $kb:
      ensure          => 'present',
      source          => $path_kb,
      install_options => ['/quiet','/IAcceptSQLServerLicenseTerms', '/Action=Patch', "/instance=${instance}"],
      require         => Mssql_secured_setup["latest_sql_server_service_packs_and_hotfixes_are_installed on ${title}"],
    }
  }
  unless(empty($sp)) {
    $path_sp = dig($update_paths, $sp)
    package { $sp:
      ensure          => 'present',
      source          => $path_sp,
      install_options => ['/quiet','/IAcceptSQLServerLicenseTerms', '/Action=Patch', "/instance=${instance}"],
      require         => Mssql_secured_setup["latest_sql_server_service_packs_and_hotfixes_are_installed on ${title}"],
    }
  }
}
