#
# mssql_secured::controls::login_auditing_is_set_to_failed_logins
#
# @summary This setting will record failed authentication attempts for SQL Server logins to the SQL Server Errorlog.
# This is the default setting for SQL Server.
# 
# Historically, this setting has been available in all versions and editions of SQL Server. Prior to the availability of SQL Server Audit, this was the only provided mechanism for capturing logins (successful or failed.
# 
# Capturing failed logins provides key information that can be used to detect\confirm password guessing attacks. Capturing successful login attempts can be used to confirm server access during forensic investigations, but using this audit level setting to also capture successful logins creates excessive noise in the SQL Server Errorlog which can hamper a DBA trying to troubleshoot problems. Elsewhere in this benchmark, we recommend using the newer lightweight SQL Server Audit feature to capture both successful and failed logins.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::login_auditing_is_set_to_failed_logins: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::login_auditing_is_set_to_failed_logins::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `login_auditing_is_set_to_failed_logins` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 5.3
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 5.3
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 5.3
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 5.3
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::login_auditing_is_set_to_failed_logins {
  $instance = $title
  $major_version = dig($facts, 'mssql_major_version', $instance)[0]

  mssql_secured_setup { "login_auditing_is_set_to_failed_logins on ${title}":
    ensure => 'present',
  }
  -> registry_value { "login_auditing_is_set_to_failed_logins on ${title}":
    ensure => present,
    path   => "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Microsoft SQL Server\\MSSQL${major_version}.${instance}\\${instance}\\AuditLevel",
    type   => 'dword',
    data   => 2,
  }
  #TODO check registry for different versions of sqlserver
}
