#
# mssql_secured::controls::maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12
#
# @summary SQL Server error log files must be protected from loss.
# The log files must be backed up before they are overwritten. Retaining more error logs helps prevent loss from frequent recycling before backups can occur.
# 
# The SQL Server error log contains important information about major server events and login attempt information as well.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 5.1
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 5.1
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 5.1
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 5.1
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Integer[12, default]] num_error_logs_hiera_value
#    This is an Integer representing the maximal number of error logs.
#    It has to be greater then 12.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12 (
  Integer[12,default] $num_error_logs_hiera_value = mssql_secured::lookup_setting('num_error_logs',undef)
) {
  $instance = $title
  $major_version = dig($facts, 'mssql_major_version', $instance)[0]

  unless($num_error_logs_hiera_value == undef) {
    $num_error_logs = $num_error_logs_hiera_value
  } else {
    $num_error_logs = 120
    warninng('No value for mssql_secured::controls::latest_sql_server_service_packs_and_hotfixes_are_installed::num_error_logs va;ue 120 is set')
  }

  mssql_secured_setup { "maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12 on ${title}":
    ensure => 'present',
  }
  -> registry_value { "maximum_number_of_error_log_files_is_set_to_greater_than_or_equal_to_12 on ${title}":
    ensure => present,
    path   => "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Microsoft SQL Server\\MSSQL${major_version}.${instance}\\${instance}\\NumErrorLogs",
    type   => 'dword',
    data   => $num_error_logs,
  }
  #TODO check registry for different versions of sqlserver
}
