#
# mssql_secured::controls::no_login_exists_with_the_name_sa
#
# @summary The sa login (e.
# g. principal) is a widely known and often widely used SQL Server account. Therefore, there should not be a login called sa even when the original sa login (principal_id = 1) has been renamed.
# 
# Enforcing this control reduces the probability of an attacker executing brute force attacks against a well-known principal name.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::no_login_exists_with_the_name_sa: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::no_login_exists_with_the_name_sa::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `no_login_exists_with_the_name_sa` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.17
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.17
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.17
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.16
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::no_login_exists_with_the_name_sa {
  mssql_secured_setup { "no_login_exists_with_the_name_sa on ${title}":
    ensure => 'present',
  }

  -> sqlserver_tsql { "no_login_exists_with_the_name_sa on ${title}":
    instance => $title,
    # lint:ignore:strict_indent
    command  => @("SQL"),
      DROP LOGIN sa
    SQL
    onlyif   => mssql_secured::exists(@(SQL)),
      SELECT principal_id, name
      FROM sys.server_principals
      WHERE name = 'sa'
    SQL
    # lint:endignore
  }

  Mssql_secured::Controls::Sa_login_account_has_been_renamed<||> -> Mssql_secured::Controls::No_login_exists_with_the_name_sa<||>
}
