#
# mssql_secured::controls::only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role
#
# @summary public is a special fixed server role containing all logins.
# Unlike other fixed server roles, permissions can be changed for the public role. In keeping with the principle of least privileges, the public server role should not be used to grant permissions at the server scope as these would be inherited by all users.
# 
# Every SQL Server login belongs to the public role and cannot be removed from this role. Therefore, any permissions granted to this role will be available to all logins unless they have been explicitly denied to specific logins or user-defined server roles.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 3.8
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 3.8
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 3.8
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.8
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role {
  mssql_secured_setup { "only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role on ${title}":
    ensure => 'present',
  }
  $instance = $title
  $extra_public_permissions = dig($facts, 'extra_public_permissions', $instance)
  unless( empty($extra_public_permissions)) {
    $extra_public_permissions.each |$permission| {
      sqlserver_tsql { "only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role on ${title}":
        instance => $title,
        # lint:ignore:strict_indent
        command  => @("SQL"),
          USE [master]
          REVOKE ${permission} FROM public;
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["only_the_default_permissions_specified_by_microsoft_are_granted_to_the_public_server_role on ${title}"],
      }
    }
  }
}
