#
# mssql_secured::controls::orphaned_users_are_dropped_from_sql_server_databases
#
# @summary A database user for which the corresponding SQL Server login is undefined or is incorrectly defined on a server instance cannot log in to the instance and is referred to as orphaned and should be removed.
# Orphan users should be removed to avoid potential misuse of those broken users in any way.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::orphaned_users_are_dropped_from_sql_server_databases: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::orphaned_users_are_dropped_from_sql_server_databases::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `orphaned_users_are_dropped_from_sql_server_databases` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 3.3
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 3.3
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 3.3
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.3
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::orphaned_users_are_dropped_from_sql_server_databases {
  $instance = $title
  $orphaned_users = dig($facts, 'mssql_sql_orphaned_users', $instance)
  mssql_secured_setup { "orphaned_users_are_dropped_from_sql_server_databases on ${title}":
    ensure => 'present',
  }
  unless( empty($orphaned_users)) {
    $orphaned_users.each | String $login | {
      sqlserver_tsql { "orphaned_users_drop on ${login} on ${title}":
        instance => $instance,
        # lint:ignore:strict_indent
        command  => @("SQL"),
          DROP LOGIN ${login} ;
        SQL
        # lint:endignore
        require  => Mssql_secured_setup["orphaned_users_are_dropped_from_sql_server_databases on ${title}"],
      }
    }
  }
}
