#
# mssql_secured::controls::public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies
#
# @summary The public database role contains every user in the msdb database.
# SQL Agent proxies define a security context in which a job step can run.
# 
# Granting access to SQL Agent proxies for the public role would allow all users to utilize the proxy which may have high privileges. This would likely break the principle of least privileges.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 3.11
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 3.11
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 3.11
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.11
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Array] exclude
#    The objects to exclude from the control actions.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies (
  Array $exclude = mssql_secured::lookup_setting('exclude', []),
) {
  mssql_secured_setup { "public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies on ${title}":
    ensure => 'present',
  }
  $instance = $title
  $sql_agent_proxies_with_public_role_in_the_msdb_database = dig($facts, 'sql_agent_proxies_with_public_role_in_the_msdb_database', $instance)
  unless (empty($sql_agent_proxies_with_public_role_in_the_msdb_database)) {
    $sql_agent_proxies_with_public_role_in_the_msdb_database.each |$proxy| {
      alert("Proxy ${proxy} is SQL Authenticated. Please consider changing it to Windows Authentication.")
      sqlserver_tsql { "sql_agent_proxy_with_public_role_in_the_msdb_database ${proxy} on ${title}":
        instance => $title,
        command  => "EXEC dbo.sp_revoke_login_from_proxy @name = N'public', @proxy_name = N'${proxy}';",
        require  => Mssql_secured_setup["public_role_in_the_msdb_database_is_not_granted_access_to_sql_agent_proxies on ${title}"],
      }
    }
  }
}
