#
# mssql_secured::controls::remote_admin_connections_server_configuration_option_is_set_to_0
#
# @summary The remote admin connections option controls whether a client application on a remote computer can use the Dedicated Administrator Connection (DAC).
# The Dedicated Administrator Connection (DAC) lets an administrator access a running server to execute diagnostic functions or Transact-SQL statements, or to troubleshoot problems on the server, even when the server is locked or running in an abnormal state and not responding to a SQL Server Database Engine connection. In a cluster scenario, the administrator may not actually be logged on to the same node that is currently hosting the SQL Server instance and thus is considered "remote". Therefore, this setting should usually be enabled (1) for SQL Server failover clusters; otherwise, it should be disabled (0) which is the default.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::remote_admin_connections_server_configuration_option_is_set_to_0: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::remote_admin_connections_server_configuration_option_is_set_to_0::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `remote_admin_connections_server_configuration_option_is_set_to_0` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.7
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.7
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.7
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.7
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::remote_admin_connections_server_configuration_option_is_set_to_0 {
  $instance = $title
  $setting  =  'remote admin connections'
  $hostname = dig($facts, 'hostname')
  $temp_dir = mssql_secured::env('TEMP')
  $system_root_dir = mssql_secured::env('SystemRoot')
  $title_without_whitespaces=regsubst($title,'\s','')
  $setting_without_whitespaces=regsubst($setting,'\s','')
  mssql_secured_setup { "remote_admin_connections_server_configuration_option_is_set_to_0 on ${title}":
    ensure => 'present',
  }
  -> file { "${temp_dir}\\dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}.ps1":
    ensure  => 'file',
    force   => true,
    content => epp('mssql_secured/dsc_sqlconfiguration_optionn_template.epp',{
        'name'            => "${title}/${setting}",
        'title'           => $title,
        'instance'        => $instance,
        'hostname'        => $hostname,
        'dsc_optionname'  => $setting,
        'dsc_optionvalue' => 0,
        'restart_service' => false
    }),
  }

  -> exec { "Exec dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}ps1":
    command  => "dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
  -> exec { "Cleanup dsc_sqlconfiguration_optionn_template${title_without_whitespaces}${setting_without_whitespaces}ps1":
    command  => "Remove-Item -Recurse -Force unnecessary_sql_server_protocols_are_set_to_disabled${title_without_whitespaces}${setting_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
}
