#
# mssql_secured::controls::sa_login_account_has_been_renamed
#
# @summary The sa account is a widely known and often widely used SQL Server login with sysadmin privileges.
# The sa login is the original login created during installation and always has principal_id=1 and sid=0x01.
# 
# It is more difficult to launch password-guessing and brute-force attacks against the sa login if the name is not known.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::sa_login_account_has_been_renamed: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::sa_login_account_has_been_renamed::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `sa_login_account_has_been_renamed` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.14
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.14
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.14
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.14
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Optional[String[1]]] preferred_value
#    The preferred value to use.
#    Some controls allow multiple values. The control implement a default value, but when you want an other value, you can use this parameter.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::sa_login_account_has_been_renamed (
  Optional[String[1]] $preferred_value = mssql_secured::lookup_setting('preferred_value', 'differt_sa'),
) {
  mssql_secured_setup { "sa_login_account_has_been_renamed on ${title}":
    ensure => 'present',
  }

  -> sqlserver_tsql { "sa_login_account_has_been_renamed on ${title}":
    instance => $title,
    # lint:ignore:strict_indent
    command  => @("SQL"),
      ALTER LOGIN sa WITH NAME = ${preferred_value}
    SQL
    onlyif   => mssql_secured::exists(@(SQL)),
      SELECT name
      FROM sys.server_principals WHERE sid = 0x01
      and name = 'sa'
    SQL
    # lint:endignore
  }
}
