#
# mssql_secured::controls::sa_login_account_is_set_to_disabled
#
# @summary The sa account is a widely known and often widely used SQL Server account with sysadmin privileges.
# This is the original login created during installation and always has the principal_id=1 and sid=0x01.
# 
# Enforcing this control reduces the probability of an attacker executing brute force attacks against a well-known principal.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::sa_login_account_is_set_to_disabled: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::sa_login_account_is_set_to_disabled::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `sa_login_account_is_set_to_disabled` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.13
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.13
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.13
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.13
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::sa_login_account_is_set_to_disabled {
  mssql_secured_setup { "sa_login_account_is_set_to_disabled on ${title}":
    ensure => 'present',
  }

  -> sqlserver_tsql { "sa_login_account_is_set_to_disabled on ${title}":
    instance => $title,
    # lint:ignore:strict_indent
    command  => @(SQL),
      DECLARE @tsql nvarchar(max)
      SET @tsql = 'ALTER LOGIN ' + SUSER_NAME(0x01) + ' DISABLE' EXEC (@tsql)
    SQL
    onlyif   => mssql_secured::exists(@(SQL)),
        SELECT name, is_disabled FROM sys.server_principals
        WHERE sid = 0x01
        AND is_disabled = 0
    SQL
    # lint:endignore
    require  => Mssql_secured_setup["sa_login_account_is_set_to_disabled on ${title}"],
  }
}
