#
# mssql_secured::controls::server_authentication_property_is_set_to_windows_authentication_mode
#
# @summary Uses Windows Authentication to validate attempted connections.
# Windows provides a more robust authentication mechanism than SQL Server authentication.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::server_authentication_property_is_set_to_windows_authentication_mode: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::server_authentication_property_is_set_to_windows_authentication_mode::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `server_authentication_property_is_set_to_windows_authentication_mode` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 3.1
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 3.1
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 3.1
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.1
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::server_authentication_property_is_set_to_windows_authentication_mode {
  $key = $facts.dig('mssql_instances', $title)

  mssql_secured_setup { "server_authentication_property_is_set_to_windows_authentication_mode on ${title}":
    ensure => 'present',
  }

  -> registry_value { "server_authentication_property_is_set_to_windows_authentication_mode on ${title}":
    ensure => present,
    path   => "HKLM\\SOFTWARE\\Microsoft\\Microsoft SQL Server\\${key}\\MSSQLServer\\LoginMode",
    type   => 'dword',
    data   => 1,
  }
}
