#
# mssql_secured::controls::sql_authentication_is_not_used_in_contained_databases
#
# @summary Contained databases do not enforce password complexity rules for SQL Authenticated users.
# The absence of an enforced password policy may increase the likelihood of a weak credential being established in a contained database.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::sql_authentication_is_not_used_in_contained_databases: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::sql_authentication_is_not_used_in_contained_databases::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `sql_authentication_is_not_used_in_contained_databases` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 3.4
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 3.4
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 3.4
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.4
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::sql_authentication_is_not_used_in_contained_databases {
  mssql_secured_setup { "sql_authentication_is_not_used_in_contained_databases on ${title}":
    ensure => 'present',
  }
  $instance = $title
  $users_that_are_using_sql_authentication = dig($facts, 'users_that_are_using_sql_authentication', $instance)
  unless (empty($users_that_are_using_sql_authentication)) {
    $users_that_are_using_sql_authentication.each |$key| {
      alert("User ${key} is SQL Authenticated. Please consider changing it to Windows Authentication.")
    }
  }
}
