#
# mssql_secured::controls::sql_server_audit_is_set_to_capture_both_failed_and_successful_logins
#
# @summary SQL Server Audit is capable of capturing both failed and successful logins and writing them to one of three places: the application event log, the security event log, or the file system.
# We will use it to capture any login attempt to SQL Server, as well as any attempts to change audit policy. This will also serve to be a second source to record failed login attempts.
# 
# By utilizing Audit instead of the traditional setting under the Security tab to capture successful logins, we reduce the noise in the ERRORLOG. This keeps it smaller and easier to read for DBAs who are attempting to troubleshoot issues with the SQL Server. Also, the Audit object can write to the security event log, though this requires operating system configuration. This gives an additional option for where to store login events, especially in conjunction with an SIEM.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::sql_server_audit_is_set_to_capture_both_failed_and_successful_logins: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::sql_server_audit_is_set_to_capture_both_failed_and_successful_logins::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `sql_server_audit_is_set_to_capture_both_failed_and_successful_logins` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 5.4
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 5.4
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 5.4
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 5.4
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::sql_server_audit_is_set_to_capture_both_failed_and_successful_logins {
  # lint:ignore:strict_indent
  $sql = mssql_secured::not_count(@(SQL),3)
    SELECT
    S.name AS 'Audit Name'
    , CASE S.is_state_enabled
    WHEN 1 THEN 'Y'
    WHEN 0 THEN 'N' END AS 'Audit Enabled'
    , S.type_desc AS 'Write Location'
    , SA.name AS 'Audit Specification Name'
    , CASE SA.is_state_enabled
    WHEN 1 THEN 'Y'
    WHEN 0 THEN 'N' END AS 'Audit Specification Enabled'
    , SAD.audit_action_name
    , SAD.audited_result
    FROM sys.server_audit_specification_details AS SAD
    JOIN sys.server_audit_specifications AS SA
    ON SAD.server_specification_id = SA.server_specification_id
    JOIN sys.server_audits AS S
    ON SA.audit_guid = S.audit_guid
    WHERE SAD.audit_action_id IN ('CNAU', 'LGFL', 'LGSD')
  SQL
  # lint:endignore
  mssql_secured_setup { "sql_server_audit_is_set_to_capture_both_failed_and_successful_logins on ${title}":
    ensure => 'present',
  }
  -> sqlserver_tsql { "sql_server_audit_is_set_to_capture_both_failed_and_successful_logins 1 on ${title}":
    instance => $title,
    # lint:ignore:strict_indent
    command  => @("SQL"),
        CREATE SERVER AUDIT TrackLogins
      TO APPLICATION_LOG
    SQL
    # lint:endignore
    onlyif   => $sql,
  }
  -> sqlserver_tsql { "sql_server_audit_is_set_to_capture_both_failed_and_successful_logins 2 on ${title}":
    instance => $title,
    # lint:ignore:strict_indent
    command  => @("SQL"),
      CREATE SERVER AUDIT SPECIFICATION TrackAllLogins
      FOR SERVER AUDIT TrackLogins
        ADD (FAILED_LOGIN_GROUP),
        ADD (SUCCESSFUL_LOGIN_GROUP),
        ADD (AUDIT_CHANGE_GROUP)
      WITH (STATE = ON)
    SQL
    # lint:endignore
    onlyif   => $sql,
  }
  -> sqlserver_tsql { "sql_server_audit_is_set_to_capture_both_failed_and_successful_logins 3 on ${title}":
    instance => $title,
    # lint:ignore:strict_indent
    command  => @("SQL"),
            ALTER SERVER AUDIT TrackLogins
      WITH (STATE = ON)
    SQL
    # lint:endignore
    onlyif   => $sql,
  }
}
