#
# mssql_secured::controls::sql_server_is_configured_to_use_non_standard_ports
#
# @summary If installed, a default SQL Server instance will be assigned a default port of TCP:1433 for TCP/IP communication.
# Administrators can also manually configure named instances to use TCP:1433 for communication. TCP:1433 is a widely known SQL Server port and this port assignment should be changed. In a multi-instance scenario, each instance must be assigned its own dedicated TCP/IP port.
# 
# Using a non-default port helps protect the database from attacks directed to the default port.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::sql_server_is_configured_to_use_non_standard_ports: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::sql_server_is_configured_to_use_non_standard_ports::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `sql_server_is_configured_to_use_non_standard_ports` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.11
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.11
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.11
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.11
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Optional[Integer]] port
#    This is the Integer containing Tcp port used by SQL Server.
#    If Undef value is passed, TcpDynamicPort setting is set to true.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::sql_server_is_configured_to_use_non_standard_ports (
  Optional[Integer] $port = mssql_secured::lookup_setting('port', undef),
) {
  $instance = $title
  $setting  =  'Sql Server Is Configured To Use Non Standard Ports'
  $hostname = dig($facts, 'hostname')
  $temp_dir = mssql_secured::env('TEMP')
  $system_root_dir = mssql_secured::env('SystemRoot')
  $title_without_whitespaces=regsubst($title,'\s','')
  $setting_without_whitespaces=regsubst($setting,'\s','')
  if $port == undef {
    $dsc_usetcpdynamicport = true
  } else {
    $dsc_usetcpdynamicport = false
  }
  mssql_secured_setup { "sql_server_is_configured_to_use_non_standard_ports on ${title}":
    ensure => 'present',
  }
  -> file { "${temp_dir}\\dsc_sqlprotocoltcpip_template${title_without_whitespaces}${setting_without_whitespaces}.ps1":
    ensure  => 'file',
    force   => true,
    content => epp('mssql_secured/dsc_sqlprotocoltcpip_template.epp',{
        'name'                  => "${title}/${setting}",
        'title'                 => $title,
        'instance'              => $instance,
        'hostname'              => $hostname,
        'dsc_ipaddressgroup'    => 'IPAll',
        'dsc_tcpport'           => $port,
        'dsc_usetcpdynamicport' => $dsc_usetcpdynamicport,
    }),
    require => Mssql_secured_setup["sql_server_is_configured_to_use_non_standard_ports on ${title}"],
  }
  -> exec { "Exec dsc_sqlprotocoltcpip_template${title_without_whitespaces}${setting_without_whitespaces}ps1":
    command  => "dsc_sqlprotocoltcpip_template${title_without_whitespaces}${setting_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
  -> exec { "Cleanup dsc_sqlprotocoltcpip_template${title_without_whitespaces}${setting_without_whitespaces}ps1":
    command  => "Remove-Item -Recurse -Force unnecessary_sql_server_protocols_are_set_to_disabled${title_without_whitespaces}${setting_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
}
