#
# mssql_secured::controls::sql_servers_sqlagent_service_account_is_not_an_administrator
#
# @summary The service account and/or service SID used by the SQLSERVERAGENT service for a default instance or SQLAGENT$<InstanceName> service for a named instance should not be a member of the Windows Administrator group either directly or indirectly (via a group).
# This also means that the account known as LocalSystem (AKA NT AUTHORITY\SYSTEM) should not be used for the SQLAGENT service as this account has higher privileges than the SQL Server service requires.
# 
# Following the principle of least privilege, the service account should have no more privileges than required to do its job. For SQL Server services, the SQL Server Setup will assign the required permissions directly to the service SID. No additional permissions or privileges should be necessary.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::sql_servers_sqlagent_service_account_is_not_an_administrator: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::sql_servers_sqlagent_service_account_is_not_an_administrator::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `sql_servers_sqlagent_service_account_is_not_an_administrator` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.6
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::sql_server_sql_agent_service_account_is_not_an_administrator {
  $instance = $title
  $setting  =  'SQL Server’s SQLAgent Service Account is Not an Administrator'
  $mssql_local_windows_administrators = dig($facts, 'mssql_local_windows_administrators')
  $mssql_domain_administrators = dig($facts, 'mssql_domain_administrators')
  $administrators = $mssql_local_windows_administrators + $mssql_domain_administrators
  $service_account = dig($facts, 'mssql_service_accounts' ).filter |$k, $v| { $v['servicename'] == 'SQL Server Agent (MSSQLSERVER)' }[0]['service_account']
  mssql_secured_setup { "sql_server_sql_agent_service_account_is_not_an_administrator on ${title}":
    ensure => 'present',
  }
  #TODO: change it from exec to group Administrators with all the members ecept mssql_service_accounnt  if ($service_account in $mssql_local_windows_administrators) {
  if ($service_account in $mssql_local_windows_administrators) {
    exec { "Remove ${service_account} from Administrators group":
      command  => "Remove-LocalGroupMember -Group 'Administrators' -Member '${service_account}'",
      provider => powershell,
      require  => Mssql_secured_setup["sql_server_sql_agent_service_account_is_not_an_administrator on ${title}"],
    }
  }
}
