#
# mssql_secured::controls::symmetric_key_encryption_algorithm_is_set_to_aes_128_or_higher_in_non_system_databases
#
# @summary Per the Microsoft Best Practices, only the SQL Server AES algorithm options, AES_128, AES_192, and AES_256, should be used for a symmetric key encryption algorithm.
# The following algorithms (as referred to by SQL Server) are considered weak or deprecated and should no longer be used in SQL Server: DES, DESX, RC2, RC4, RC4_128.
# 
# Many organizations may accept the Triple DES algorithms (TDEA) which use keying options 1 (3 key aka 3TDEA) or keying option 2 (2 key aka 2TDEA). In SQL Server, these are referred to as TRIPLE_DES_3KEY and TRIPLE_DES respectively. Additionally, the SQL Server algorithm named DESX is actually the same implementation as the TRIPLE_DES_3KEY option. However, using the DESX identifier as the algorithm type has been deprecated and its usage is now discouraged.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::symmetric_key_encryption_algorithm_is_set_to_aes_128_or_higher_in_non_system_databases: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::symmetric_key_encryption_algorithm_is_set_to_aes_128_or_higher_in_non_system_databases::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `symmetric_key_encryption_algorithm_is_set_to_aes_128_or_higher_in_non_system_databases` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 7.1
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 7.1
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 7.1
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 7.1
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::symmetric_key_encryption_algorithm_is_set_to_aes_128_or_higher_in_non_system_databases {
  mssql_secured_setup { "symmetric_key_encryption_algorithm_is_set_to_aes_128_or_higher_in_non_system_databases on ${title}":
    ensure => 'present',
  }
  $instance = $title
  $symetric_keys = dig($facts, 'mssql_symetric_keys_in_non_system_databases', $instance)
  unless (empty($symetric_keys)) {
    $symetric_keys.each |$key| {
      alert("Symetric key ${key} length is less then 2048 bits. Use another mey with length 2048 or more.")
    }
  }
}
