#
# mssql_secured::controls::trustworthy_database_property_is_set_to_off
#
# @summary The TRUSTWORTHY database option allows database objects to access objects in other databases under certain circumstances.
# Provides protection from malicious CLR assemblies or extended procedures.
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::trustworthy_database_property_is_set_to_off: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::trustworthy_database_property_is_set_to_off::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `trustworthy_database_property_is_set_to_off` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2014 CIS V1.5.0](/docs/mssql_secured/cis/mssql2014_V1.5.0.html) - paragraph 2.9
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 2.9
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 2.9
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.9
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Array] exclude
#    The objects to exclude from the control actions.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::trustworthy_database_property_is_set_to_off (
  Array $exclude = mssql_secured::lookup_setting('exclude', []),
) {
  $instance = $title

  mssql_secured_setup { "trustworthy_database_property_is_set_to_off on ${title}":
    ensure => 'present',
  }
  $databases_to_check = dig($facts, 'mssql_databases', $instance) - ['msdb'] - $exclude
  unless( empty($databases_to_check)) {
    $databases_to_check.each |$database| {
      sqlserver_tsql { "trustworthy_database_property_is_set_to_off on ${instance}/${database}":
        instance => $instance,
        command  => "ALTER DATABASE ${database} SET TRUSTWORTHY OFF;",
        onlyif   => mssql_secured::exists("SELECT name FROM sys.databases WHERE is_trustworthy_on = 1 AND name = '${database}'"),
        require  => Mssql_secured_setup["trustworthy_database_property_is_set_to_off on ${title}"],
      }
    }
  }
}
