#
# mssql_secured::controls::unnecessary_sql_server_protocols_are_set_to_disabled
#
# @summary SQL Server supports Shared Memory, Named Pipes, and TCP/IP protocols.
# However, SQL Server should be configured to use the bare minimum required based on the organization's needs.
# 
# Using fewer protocols minimizes the attack surface of SQL Server and, in some cases, can protect it from remote attacks.
# 
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::unnecessary_sql_server_protocols_are_set_to_disabled: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::unnecessary_sql_server_protocols_are_set_to_disabled::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `unnecessary_sql_server_protocols_are_set_to_disabled` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 2.10
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
# @param [Optional[Boolean]] tcp_enabled
#    This is the Booleand value containing information if Tcp protocol is enabled in the SQL Server instance.
#
# @param [Optional[Boolean]] np_enabled
#    This is the Booleand value containing information if Np protocol is enabled in the SQL Server instance.
#
# @param [Optional[Boolean]] sm_enabled
#    This is the Booleand value containing information if SM protocol is enabled in the SQL Server instance.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::unnecessary_sql_server_protocols_are_set_to_disabled (
  Optional[Boolean] $tcp_enabled = mssql_secured::lookup_setting('tcp_enabled', undef),
  Optional[Boolean] $np_enabled = mssql_secured::lookup_setting('np_enabled', undef),
  Optional[Boolean] $sm_enabled = mssql_secured::lookup_setting('sm_enabled', undef),
) {
  $instance = $title
  $hostname = dig($facts, 'hostname')
  $temp_dir = mssql_secured::env('TEMP')
  $system_root_dir = mssql_secured::env('SystemRoot')
  $title_without_whitespaces=regsubst($title,'\s','')
  mssql_secured_setup { "unnecessary_sql_server_protocols_are_set_to_disabled on ${title}":
    ensure => 'present',
  }
  -> file { "${temp_dir}\\unnecessary_sql_server_protocols_are_set_to_disabled${title}.ps1":
    ensure  => 'file',
    force   => true,
    content => epp('mssql_secured/sql_server_protocols_template.epp',{
        'instance'    => $instance,
        'hostname'    => $hostname,
        'tcp_enabled' => $tcp_enabled,
        'np_enabled'  => $np_enabled,
        'sm_enabled'  => $sm_enabled,
    }),
  }
  -> exec { "Exec unnecessary_sql_server_protocols_are_set_to_disabled${title}ps1":
    command  => "unnecessary_sql_server_protocols_are_set_to_disabled${title_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
  -> exec { "Cleanup unnecessary_sql_server_protocols_are_set_to_disabled${title}ps1":
    command  => "Remove-Item -Recurse -Force unnecessary_sql_server_protocols_are_set_to_disabled${title_without_whitespaces}.ps1",
    path     => $temp_dir,
    provider => pwsh,
  }
}
