#
# mssql_secured::controls::windows_local_groups_are_not_sql_logins
#
# @summary Local Windows groups should not be used as logins for SQL Server instances.
# Allowing local Windows groups as SQL Logins provides a loophole whereby anyone with OS level administrator rights (and no SQL Server rights) could add users to the local Windows groups and thereby give themselves or others access to the SQL Server instance.
# 
# 
# ## Skipping
# 
# To deliberately skip this control (e.g. meaning don't use Puppet to enforce this setting), we provide
# you with three ways:
# 
# 1) Add `mssql_secured::controls::windows_local_groups_are_not_sql_logins: skip` to your hiera data. This will skip this control for **ALL** databases.  
# 2) Add `mssql_secured::controls::windows_local_groups_are_not_sql_logins::dbname: skip` to your hiera data. This will skip this control for specified database only.  
# 3) Add an entry with the content `windows_local_groups_are_not_sql_logins` to the array value `mssql_secured::skip_list` in your hiera data.  
# 
# 
# 
# ## Benchmarks
# 
# This control is used in the following benchmarks:
# 
# - [Microsoft SQL Server Database 2016 CIS V1.3.0](/docs/mssql_secured/cis/mssql2016_V1.3.0.html) - paragraph 3.10
# - [Microsoft SQL Server Database 2017 CIS V1.2.0](/docs/mssql_secured/cis/mssql2017_V1.2.0.html) - paragraph 3.10
# - [Microsoft SQL Server Database 2019 CIS V1.2.0](/docs/mssql_secured/cis/mssql2019_V1.2.0.html) - paragraph 3.10
#
# @param title
#    The database to apply the control to.
#    All controls need an database to apply the control to. Here is a simple example:
#    ``` puppet
#    mssql_secured::controls::control_name {'MSSQLSERVER':}
#    ```
#    In this example, the string DB is the database to apply the control to.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mssql_secured::controls::windows_local_groups_are_not_sql_logins {
  $instance = $title
  $local_group_logins = dig($facts, 'mssql_local_groups_logins' , $instance)
  mssql_secured_setup { "windows_local_groups_are_not_sql_logins on ${title}":
    ensure => 'present',
  }
  unless( empty($local_group_logins)) {
    $local_group_logins.each |$login| {
      alert("${login} is Windows local group. Create a more restrictive AD group containing only the required user accounts.")
    }
  }
}
